/**
 * Denne filen inneholder koden for en server som jobber som et mellomledd mellom NTNU apiet og applikasjonen
 * 
 * @file api_handler.js
 */

/* Code taken from: https://github.com/kubowania/crypto-dashboard-react/blob/main/package.json */
const PORT = 8080;
const express = require('express');
const cors = require('cors');
const axios = require('axios');
var bodyParser = require('body-parser');
const { authenticate } = require('ldap-authentication');
const crypto = require('crypto');
const { stat } = require('fs');
const { response } = require('express');
const { count } = require('console');
require('dotenv').config();

const app = express();

app.use(cors());

app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: true }))

/*Available endpoints:
/alert
/content
/dhcp
/flow
/dns
/ikev2
/krb
/rdp
/sip
/smtp
/ssh
/stats
/tls
*/

/**
 * Alle API endepunktene har den samme koden for å fetche dataen, bare at de gaar til forskjellige endepunkt.
 */


/*****************************
 ****ALLE API ENDEPUNKTENE****
 *****************************/
app.get('/alert', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/alert',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/content', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/content',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/dhcp', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/dhcp',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/flow', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/flow',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/dns', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/dns',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/ikev2', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/ikev2',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/krb', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/krb',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/rdp', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/rdp',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/sip', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/sip',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/smtp', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/smtp',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/ssh', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/ssh',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/stats', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/stats',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});

app.get('/tls', (req, res) => {
    const options = {
        method: 'GET',
        url: 'https://apitest01.soc.ntnu.no:8082/surtest/tls',
        headers: {
            'X-Gravitee-Api-Key': process.env.API_KEY,
        },
    }
    axios.request(options).then((response) => {
        let data = response.data;
            data = data.replaceAll('\n', ',');
            data = '[' + data;
            data = data.slice(0, data.length - 1);
            data = data + ']';
        res.json(data);
    }).catch((err) => console.log(err))
});
/*****************************
 **** API ENDEPUNKT SLUTT ****
 *****************************/

/**
 * Inneholder kode fra:
 * //https://www.npmjs.com/package/ldap-authentication
 * 
 * Gjør en spørring på NTNU sitt innloggings API med brukeren sin innformasjon
 * 
 * @param {*} credentials Brukeren sitt brukernavn og passord
 * @returns True eller false basert på om brukeren blir autentisert eller ikke
 */
async function auth(credentials) {
    //Variablene som blir sendt med i spørringen
    let options = {
        ldapOpts: { 
            url: "ldaps://at.ntnu.no",
        },
        userDn: `uid=${credentials.username},ou=people,dc=ntnu,dc=no`,
        userPassword: credentials.password,
        userSearchBase: 'ou=people,dc=ntnu,dc=no',
        usernameAttribute: 'uid',
        username: credentials.username,
    }

    try {
        let user = await authenticate(options);
        //Blir kun autorisert om brukeren er med i IT-sikkerhetsavdelingen
        if (user.ntnuMemberOf.find(element => element === "od-it-sikkerhet_soc")) {
            console.log(user);
            return true; }
        else {
            console.log("Ingen bruker funnet");
            return false }
    }
    catch (e) {
        console.log(e.message)
    }
}

/**
 * Avventer om spørringen på NTNU APIet blir godkjent, hvis den blir det
 * så blir det returnert en token som blir lagret i databasen og sendt
 * oppover til brukeren.
 */
app.post('/login', (req, res) => {
    (async () => {
        if (await auth(req.body)) {
            let gToken = crypto.randomBytes(64).toString('hex')
            saveToken(gToken);
            res.send({
                token: gToken
              });
        };
    })()
}) 

/**
 * Gir UNIX tiden til serveren og returnerer den
 */
app.get('/time', (req, res) => {
    let time = Math.floor(Date.now() / 1000)

    console.log(time);
    
    res.end(String(time));
})

/**
 * Henter tokenen til brukeren og sjekker om den samsvarer med en som ligger i databasen
 */
app.post('/getToken', (req, res) => {
    let parsedToken = req.body.token;
    let time = Math.floor(Date.now() / 1000);

    let options = {
        method: 'GET',
        url: 'http://localhost:8000/api/tokens/',
    }
    
    //Sjekker om tokenen er gyldig, både stringen og tiden som sjekken blir gjort på
    axios.request(options).then((response) => {
        if (response.data.find(e => { return ( e.token === parsedToken && time <= e.timestamp + parseInt(process.env.TOKEN_DURATION))})) {
            console.log("VALID TOKEN");
            res.send(true);
        }
        else {
            console.log("INVALID TOKEN")
            res.send(false);
        } 
    }).catch((err) => console.log(err))
})

/**
 * Lagrer tokenen i databasen
 * 
 * @param {*} token Tokenen som skal bli lagret
 */
async function saveToken (token) {
    let options = {
        method: 'POST',
        url: "http://localhost:8000/api/tokens/",
        data: {
            token: token,
            timestamp: Math.floor(Date.now() / 1000)
        }
    }

    axios.request(options)
    .then((res) => {
        console.log(res.status)
        status = res.status;
    })
    .catch((err) => console.log(err))
}

// Sets up the server to listen
app.listen(PORT, () => console.log(`Server is running on port ${PORT}`))