/**
 * @file App.js 
 */
import './App.css';
import 'bootstrap/dist/css/bootstrap.min.css';
import Toolbar from "./Toolbar";
import AdminUI from "./AdminUI";
import PresentationUI from "./PresentationUI";
import Home from './Home';
import { BrowserRouter as Router, Route, Routes } from 'react-router-dom';
import 'react-notifications-component/dist/theme.css';
import Login from './login'
import { useToken } from './tokenHandling'

/**
 * @description Main component of the web page
 * @returns JSX code that contains all the sub-components
 */
export default function App() {
  const {token, setToken} = useToken();

  if (!token) {
    return <Login setToken={setToken} />
  }
  
  return (
    <Router>
        <div className="App">
            <Toolbar />
            <div className="middle">
              <Routes>
                <Route path="/" element={<Home />} /> 
                <Route path="/admin/:id" element={<AdminUI/>} />
                <Route path="/window/:id" element={<PresentationUI/>} />
              </Routes>
            </div>   
        </div>
    </Router>
  );
}
