import axios from 'axios';
import { useEffect, useState } from 'react';
import redTriangle from './images/warning.png';
import yellowTriangle from './images/yellow_warning.png';
import { Store } from 'react-notifications-component';
import { Modal, Container, Row, Col } from "react-bootstrap";

const Event = () => {
    const [events, setEvents] = useState([]);
    const [modal, setModal] = useState(false);

    const modalOpen = () => {
        setModal(true);
    }

    const modalClose = () => {
        setModal(false);
    }

    const unseenEvent = () => {
        for(let i = 0; i < events.length; i++)
            if(!events[i].seen)
                return true;
        return false;
    }

    useEffect(() => {
        const interval = setInterval(() => {
            axios
            .get('http://localhost:8000/api/events/')
            .then((res) => { 
                let newEvents = res.data;
                let oldEvents = events;
                if(oldEvents != newEvents && newEvents.length > 0)
                {
                    if(oldEvents.length != 0)
                    {
                        let idx = newEvents.findIndex((e) => (e.id == oldEvents[oldEvents.length - 1].id) );
                        let unpushedEvents = newEvents.slice(idx);
                        unpushedEvents.map((e) => {
                            if(!e.seen)
                                createNotification(e.id, 'warning', e.title, e.description);
                        });
                    }
                    else {
                        newEvents.map((e) => {
                            if(!e.seen)
                                createNotification(e.id, 'warning', e.title, e.description);
                        });
                    }
                }
                setEvents(newEvents);
            })
            .catch((err) => console.log(err))
        }, 10000); // Updates every 10 seconds
        return () => clearInterval(interval);
    }, []);

    const addSeen = (id) => {
        console.log(id);
        let idx = events.findIndex((e) => 
        {
            console.log(e);
            return e.id == id;
        });
        console.log(idx);
        if(idx >= 0) {
            events[idx].seen = true;
            axios.put('http://localhost:8000/api/events/' + events[idx].id + '/', events[idx])
            setEvents(events);
        }
    }

    const createNotification = (id, typ, tit, des) => {
        console.log('Notification');
        Store.addNotification({
            id: id,
            title: tit,
            message: des,
            type: typ,
            insert: "top",
            container: "top-right",
            animationIn: ["animate__animated", "animate__fadeIn"],
            animationOut: ["animate__animated", "animate__fadeOut"],
            dismiss: {
              duration: 0,
              onScreen: true
            },
            onRemoval: addSeen,
          });
    }

    const renderEvent = (e) => {
        return(
            <Row>
                <Col lg={2} md={2}>
                    {e.title}
                </Col>
                <Col>
                    {e.description}
                </Col>
                <Col lg={1} md={1}>
                    {e.seen}
                </Col>
            </Row>
        );
    }

    return ( 
    <div>
        {
        (unseenEvent()) ? 
        <img src={redTriangle} width={25} height={25} onClick={modalOpen} /> //<a href="https://www.flaticon.com/free-icons/error" title="error icons">Error icons created by Freepik - Flaticon</a> 
        : <img src={yellowTriangle} width={25} height={25} onClick={modalOpen} /> //<a href="https://www.flaticon.com/free-icons/warning" title="warning icons">Warning icons created by Good Ware - Flaticon</a>
        }
        <Modal show={modal} fullscreen={true} onHide={modalClose}>
            <Modal.Header closeButton>
                <Modal.Title>Event modal</Modal.Title>
            </Modal.Header>
            <Modal.Body className='show-grid'>
                <Container>
                    {events.map((e) => renderEvent(e))}
                </Container>
            </Modal.Body>
            <Modal.Footer>
                something else
            </Modal.Footer>
        </Modal>
    </div>
    );
}
 
export default Event;