/**
 * @description Skal returnere en react komponent som inneholder toolbaren og dens funksjonalitet.
 * 
 * @file Toolbar.js
 */

import './Toolbar.css';
import logo from './images/ntnu_neg.png';
import Clock from './Clock';
import { deleteToken } from './tokenHandling'
import { useLocation, BrowserRouter as Link } from 'react-router-dom';
import { useEffect } from 'react';
import { checkToken, useToken } from './tokenHandling';


/**
 * React komponent for toolbaren, sjekker om tokens er gyldig og
 * definerer hvilken funksjonalitet toolbaren skal ha.
 * 
 * @returns JSX kode
 */
const Toolbar = () => {
    const location = useLocation();
    const {token, setToken} = useToken(); ///< Tokenen som skal bli lagret

    //Sjekker om tokenen er gyldig etter komponenten er lastet inn
    useEffect(() => {
       checkToken(token);
    }, [location]);

    return( 
        <nav className="toolbar">
            <div className="navleft">
                <Link to="/">
                    <img src={logo}></img>
                </Link>
            </div>
            <div className="navright">
                <Clock />
                <button onClick={() => deleteToken()}>Log out</button>
            </div>
        </nav>
     );
}
 
export default Toolbar;